@@ -1,7 +1,8 @@
 import re
+import warnings
 import numpy as np
 import pandas as pd
-from ..Utils import clean_keys
+from Amplo.Utils import clean_keys
 
 
 class DataProcesser:
@@ -140,6 +141,9 @@ class DataProcesser:
         # Clean Keys
         data = clean_keys(data)
 
+        # Impute columns
+        data = self._impute_columns(data)
+
         # Remove duplicates
         data = self.remove_duplicates(data, rows=False)
 
@@ -164,6 +168,8 @@ class DataProcesser:
         assert self.is_fitted, "Object not yet fitted."
         return {
             'num_cols': self.num_cols,
+            'float_cols': self.float_cols,
+            'int_cols': self.int_cols,
             'date_cols': self.date_cols,
             'cat_cols': self.cat_cols,
             'missing_values': self.missing_values,
@@ -180,17 +186,19 @@ class DataProcesser:
         """
         Loads settings from dictionary and recreates a fitted object
         """
-        self.num_cols = settings['num_cols']
-        self.cat_cols = settings['cat_cols']
-        self.date_cols = settings['date_cols']
-        self.missing_values = settings['missing_values']
-        self.outlier_removal = settings['outlier_removal']
-        self.z_score_threshold = settings['z_score_threshold']
+        self.num_cols = settings['num_cols'] if 'num_cols' in settings else []
+        self.float_cols = settings['float_cols'] if 'float_cols' in settings else []
+        self.int_cols = settings['int_cols'] if 'int_cols' in settings else []
+        self.cat_cols = settings['cat_cols'] if 'cat_cols' in settings else []
+        self.date_cols = settings['date_cols'] if 'date_cols' in settings else []
+        self.missing_values = settings['missing_values'] if 'num_cols' in settings else []
+        self.outlier_removal = settings['outlier_removal'] if 'num_cols' in settings else []
+        self.z_score_threshold = settings['z_score_threshold'] if 'num_cols' in settings else []
         self._means = None if settings['_means'] is None else pd.read_json(settings['_means'])
         self._stds = None if settings['_stds'] is None else pd.read_json(settings['_stds'])
         self._q1 = None if settings['_q1'] is None else pd.read_json(settings['_q1'])
         self._q3 = None if settings['_q3'] is None else pd.read_json(settings['_q3'])
-        self.dummies = settings['dummies']
+        self.dummies = settings['dummies'] if 'dummies' in settings else {}
         self.is_fitted = True
 
     def infer_data_types(self, data: pd.DataFrame):
@@ -245,12 +253,19 @@ class DataProcesser:
         for key in self.date_cols:
             data.loc[:, key] = pd.to_datetime(data[key], errors='coerce', infer_datetime_format=True, utc=True)
 
-        # Numerical columns
-        for key in [key for key in data.keys() if key not in self.date_cols and key not in self.cat_cols]:
-            if pd.api.types.is_float_dtype(data[key]):
-                data.loc[:, key] = pd.to_numeric(data[key], errors='coerce', downcast='float')
-            elif pd.api.types.is_integer_dtype(data[key]):
-                data.loc[:, key] = pd.to_numeric(data[key], errors='coerce', downcast='integer')
+        # Integer columns
+        for key in self.int_cols:
+            data.loc[:, key] = pd.to_numeric(data[key], errors='coerce', downcast='integer')
+
+        # Float columns
+        for key in self.float_cols:
+            data.loc[:, key] = pd.to_numeric(data[key], errors='coerce', downcast='float')
+
+        # Numeric columns that are not in Float or Integer -- redundancy
+        for key in self.num_cols:
+            if key in self.float_cols + self.int_cols:
+                continue
+            data.loc[:, key] = pd.to_numeric(data[key], errors='coerce')
 
         # Categorical columns
         if fit_categorical:
@@ -260,6 +275,12 @@ class DataProcesser:
                                    "encoder is not yet fitted."
             data = self._transform_cat_cols(data)
 
+        # We need everything to become numeric, so all that is not mentioned will be handled as numeric
+        all_cols = self.float_cols + self.int_cols + self.num_cols + self.date_cols + self.cat_cols
+        for key in data.keys():
+            if key not in all_cols:
+                data.loc[:, key] = pd.to_numeric(data[key], errors='coerce')
+
         return data
 
     def _fit_cat_cols(self, data: pd.DataFrame) -> pd.DataFrame:
@@ -303,8 +324,10 @@ class DataProcesser:
         data = data.loc[:, ~data.columns.duplicated()]
 
         # Note
-        self.removedDuplicateColumns = len(data.keys()) - n_columns
-        self.removedDuplicateRows = len(data) - n_rows
+        self.removedDuplicateColumns = n_columns - len(data.keys())
+        self.removedDuplicateRows = n_rows - len(data)
+        print(f'[AutoML] Removed {self.removedDuplicateColumns} duplicate columns and {self.removedDuplicateRows} '
+              f'duplicate rows')
 
         return data
 
@@ -319,6 +342,7 @@ class DataProcesser:
 
         # Note
         self.removedConstantColumns = columns - len(data.keys())
+        print(f'[AutoML] Removed {self.removedConstantColumns} constant columns.')
 
         return data
 
@@ -378,6 +402,7 @@ class DataProcesser:
 
         # Note
         self.imputedMissingValues = data[self.num_cols].isna().sum().sum()
+        print(f'[AutoML] Imputed {self.imputedMissingValues} missing values.')
 
         # Removes all rows with missing values
         if self.missing_values == 'remove_rows':
@@ -455,3 +480,17 @@ class DataProcesser:
                     for i, val in enumerate(sorted(set(data[self.target]))):
                         data.loc[data[self.target] == val, self.target] = i
         return data
+
+    def _impute_columns(self, data: pd.DataFrame) -> pd.DataFrame:
+        """
+        *** For production ***
+        If a dataset is missing certain columns, this function looks at all registered columns and fills them with
+        zeros.
+        """
+        imputed = []
+        for keys in [self.num_cols, self.date_cols, self.cat_cols]:
+            for key in [k for k in keys if k not in data]:
+                data[key] = np.zeros(len(data))
+                imputed.append(key)
+        warnings.warn(f'Imputed {len(imputed)} missing columns! {imputed}')
+        return data
