@@ -160,7 +160,7 @@ class FeatureProcesser:
         assert self.is_fitted, "Can only use transform after object is fitted."
 
         # Clean & downcast data
-        data = data.astype('float32').clip(lower=1e-12, upper=1e12).fillna(0)
+        data = data.astype('float32').clip(lower=-1e12, upper=1e12).fillna(0)
 
         # Get original features
         features = self.featureSets[feature_set]
@@ -178,11 +178,11 @@ class FeatureProcesser:
         required = copy.copy(original_features)
         required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.linearFeatures]))
         required += list(itertools.chain.from_iterable([s.split('__')[::2] for s in self.crossFeatures]))
-        required += list(itertools.chain.from_iterable([s.split('__')[1] for s in self.trigonometricFeatures]))
-        required += list(itertools.chain.from_iterable([s[5:] for s in self.inverseFeatures]))
-        required += list(itertools.chain.from_iterable([s.split('__diff__')[0] for s in self.diffFeatures]))
-        required += list(itertools.chain.from_iterable([s.split('__lag__')[0] for s in self.laggedFeatures]))
-        required += list(itertools.chain.from_iterable([s.split('__dt__')[0] for s in self.datetimeFeatures]))
+        required += [s.split('__')[1] for s in self.trigonometricFeatures]
+        required += [s[5:] for s in self.inverseFeatures]
+        required += [s.split('__diff__')[0] for s in self.diffFeatures]
+        required += [s.split('__lag__')[0] for s in self.laggedFeatures]
+        required += [s.split('__dt__')[0] for s in self.datetimeFeatures]
         if len(self.kMeansFeatures) != 0:
             required += list(self._centers.keys())
 
@@ -192,7 +192,7 @@ class FeatureProcesser:
         # Impute missing keys
         missing_keys = [k for k in required if k not in data.keys()]
         if len(missing_keys) > 0:
-            warnings.warn('[Feature Extraction] Imputing {} keys'.format(len(missing_keys)))
+            warnings.warn('[AutoML] Imputing {} keys: {}'.format(len(missing_keys), missing_keys))
         for k in missing_keys:
             data.loc[:, k] = np.zeros(len(data))
 
@@ -227,7 +227,7 @@ class FeatureProcesser:
         self.x = self.x[features]
 
         # And clip everything (we do this with all features in ._analyse_feature(), no exception)
-        self.x = self.x.astype('float32').clip(lower=1e-12, upper=1e12).fillna(0)
+        self.x = self.x.astype('float32').clip(lower=-1e12, upper=1e12).fillna(0)
 
         return self.x
 
@@ -254,17 +254,17 @@ class FeatureProcesser:
         """
         Loads settings from dictionary and recreates a fitted object
         """
-        self.linearFeatures = settings['linearFeatures']
-        self.crossFeatures = settings['crossFeatures']
-        self.trigonometricFeatures = settings['trigonometricFeatures']
-        self.inverseFeatures = settings['inverseFeatures']
-        self.kMeansFeatures = settings['kMeansFeatures']
-        self.laggedFeatures = settings['laggedFeatures']
-        self.diffFeatures = settings['diffFeatures']
-        self.featureSets = settings['featureSets']
-        self._means = pd.read_json(settings['_means'], typ='series')
-        self._stds = pd.read_json(settings['_stds'], typ='series')
-        self._centers = pd.read_json(settings['_centers'])
+        self.linearFeatures = settings['linearFeatures'] if 'linearFeatures' in settings else []
+        self.crossFeatures = settings['crossFeatures'] if 'crossFeatures' in settings else []
+        self.trigonometricFeatures = settings['trigonometricFeatures'] if 'trigonometricFeatures' in settings else []
+        self.inverseFeatures = settings['inverseFeatures'] if 'inverseFeatures' in settings else []
+        self.kMeansFeatures = settings['kMeansFeatures'] if 'kMeansFeatures' in settings else []
+        self.laggedFeatures = settings['laggedFeatures'] if 'laggedFeatures' in settings else []
+        self.diffFeatures = settings['diffFeatures'] if 'diffFeatures' in settings else {}
+        self.featureSets = settings['featureSets'] if 'featureSets' in settings else []
+        self._means = pd.read_json(settings['_means'], typ='series') if 'v' in settings else []
+        self._stds = pd.read_json(settings['_stds'], typ='series') if '_stds' in settings else []
+        self._centers = pd.read_json(settings['_centers']) if '_centers' in settings else []
         self.is_fitted = True
 
     def _clean_set(self, x: pd.DataFrame, y: pd.Series) -> None:
@@ -280,7 +280,7 @@ class FeatureProcesser:
             self.model = DecisionTreeRegressor(max_depth=3)
 
         # Bit of necessary data cleaning (shouldn't change anything)
-        x = clean_keys(x.astype('float32').clip(lower=1e-12, upper=1e12).fillna(0).reset_index(drop=True))
+        x = clean_keys(x.astype('float32').clip(lower=-1e12, upper=1e12).fillna(0).reset_index(drop=True))
         self.x = copy.copy(x)
         self.originalInput = copy.copy(x)
         self.y = y.replace([np.inf, -np.inf], 0).fillna(0).reset_index(drop=True)
@@ -300,7 +300,7 @@ class FeatureProcesser:
         In case of multiclass, we score per class :)
         """
         # Clean feature
-        feature = feature.clip(lower=1e-12, upper=1e12).fillna(0).values.reshape((-1, 1))
+        feature = feature.clip(lower=-1e12, upper=1e12).fillna(0).values.reshape((-1, 1))
 
         # Copy & fit model
         m = copy.copy(self.model)
@@ -417,7 +417,7 @@ class FeatureProcesser:
         # Add features
         for k in self.datetimeFeatures:
             key, _, period = k.split('__')
-            self.x[k] = getattr(self.x[key].dt, period).clip(lower=1e-12, upper=1e12).fillna(0)
+            self.x[k] = getattr(self.x[key].dt, period).clip(lower=-1e12, upper=1e12).fillna(0)
 
         # Print result
         if self.verbosity > 0:
@@ -480,11 +480,11 @@ class FeatureProcesser:
             if '__x__' in k:
                 key_a, key_b = k.split('__x__')
                 feature = self.x[key_a] * self.x[key_b]
-                self.x[k] = feature.clip(lower=1e-12, upper=1e12).fillna(0)
+                self.x[k] = feature.clip(lower=-1e12, upper=1e12).fillna(0)
             else:
                 key_a, key_b = k.split('__d__')
                 feature = self.x[key_a] / self.x[key_b]
-                self.x[k] = feature.clip(lower=1e-12, upper=1e12).fillna(0)
+                self.x[k] = feature.clip(lower=-1e12, upper=1e12).fillna(0)
 
         # Print result
         if self.verbosity > 0:
@@ -581,11 +581,11 @@ class FeatureProcesser:
             if '__sub__' in key:
                 key_a, key_b = key.split('__sub__')
                 feature = self.x[key_a] - self.x[key_b]
-                self.x.loc[:, key] = feature.clip(lower=1e-12, upper=1e12).fillna(0)
+                self.x.loc[:, key] = feature.clip(lower=-1e12, upper=1e12).fillna(0)
             else:
                 key_a, key_b = key.split('__add__')
                 feature = self.x[key_a] + self.x[key_b]
-                self.x.loc[:, key] = feature.clip(lower=1e-12, upper=1e12).fillna(0)
+                self.x.loc[:, key] = feature.clip(lower=-1e12, upper=1e12).fillna(0)
 
         # store
         if self.verbosity > 0:
@@ -643,7 +643,7 @@ class FeatureProcesser:
         k_means = MiniBatchKMeans(n_clusters=clusters)
         column_names = ['dist__{}_{}'.format(i, clusters) for i in range(clusters)]
         distances = pd.DataFrame(columns=column_names, data=k_means.fit_transform(data))
-        distances = distances.clip(lower=1e-12, upper=1e12).fillna(0)
+        distances = distances.clip(lower=-1e12, upper=1e12).fillna(0)
         self._centers = pd.DataFrame(columns=self.originalInput.keys(), data=k_means.cluster_centers_)
 
         # Analyse correlation
@@ -660,18 +660,20 @@ class FeatureProcesser:
         """
         Adds the selected k-means features
         """
-        # Normalize data
-        tmp = copy.deepcopy(self.x.loc[:, self._centers.keys()])
-        tmp -= self._means
-        tmp /= self._stds
+        # Only execute if necessary
+        if len(self.kMeansFeatures) > 0:
+            # Normalize data
+            tmp = copy.deepcopy(self.x.loc[:, self._centers.keys()])
+            tmp -= self._means
+            tmp /= self._stds
 
-        # Calculate centers
-        for key in self.kMeansFeatures:
-            ind = int(key[key.find('dist__') + 6: key.rfind('_')])
-            self.x.loc[:, key] = np.sqrt(np.square(tmp - self._centers.iloc[ind]).sum(axis=1))
+            # Calculate centers
+            for key in self.kMeansFeatures:
+                ind = int(key[key.find('dist__') + 6: key.rfind('_')])
+                self.x.loc[:, key] = np.sqrt(np.square(tmp - self._centers.iloc[ind]).sum(axis=1))
 
-        if self.verbosity > 0:
-            print('[AutoML] Added {} K-Means features'.format(len(self.kMeansFeatures)))
+            if self.verbosity > 0:
+                print('[AutoML] Added {} K-Means features'.format(len(self.kMeansFeatures)))
 
     def _fit_diff_features(self):
         """
@@ -711,7 +713,7 @@ class FeatureProcesser:
             key, diff = k.split('__diff__')
             feature = self.x[key]
             for i in range(1, int(diff)):
-                feature = feature.diff().clip(lower=1e-12, upper=1e12).fillna(0)
+                feature = feature.diff().clip(lower=-1e12, upper=1e12).fillna(0)
             self.x[k] = feature
 
         # Print output
